



from pandasflow.metrics.mean_error import mean_error

from pandasflow.metrics.lloss_up import lloss_up

from pandasflow.metrics.conf_mat import conf_mat

from pandasflow.metrics.best_f1 import best_f1

from pandasflow.metrics.precision_recall import precision_recall

from pandasflow.metrics.roc import roc


import pandasflow as pdf
import pandas as pd
from sklearn.metrics import precision_recall_curve, average_precision_score
from matplotlib import pyplot


def precision_recall(target, score, plot=False, marker='', round_=2):
	
	if type(score) == pd.Series or type(score) == list and len(score) == 1:
		score_ = score if type(score) == pd.Series else score[0]
		score_name = score_.name
		
		precision, recall, _ = precision_recall_curve(target, score_)
		pyplot.plot(recall, precision, marker=marker, label=score_name)
		
		average = average_precision_score(target, score_)
		average = round(average, round_)
		print(f"{score_name} average precision: ", average)
	
	
	if type(score) == list and len(score) > 1:
		for sc in score:
			precision, recall, _ = precision_recall_curve(target, sc)
			pyplot.plot(recall, precision, marker=marker, label=sc.name)
			
			average = average_precision_score(target, sc)
			average = round(average, round_)
			print(f"{sc.name} average precision: ", average)
			
	pyplot.xlabel('Recall')
	pyplot.ylabel('Precision')
	pyplot.legend()
	
	if plot:
		pyplot.show()


import pandasflow as pdf
import pandas as pd
import numpy as np





t = pd.DataFrame({'y_true': [0]*990 + [1]*10})

np.random.seed(42)

t['y_score1'] = np.concatenate([np.random.uniform(0, 0.2, size=970),
                                np.random.uniform(0.8, 1, size=20),
                                np.random.uniform(0, 0.2, size=5),
                                np.random.uniform(0.8, 1, size=5)])

t['y_score2'] = np.concatenate([np.random.uniform(0, 0.2, size=900), #TN
                                np.random.uniform(0.8, 1, size=90),  #FP
                                np.random.uniform(0, 0.2, size=4),   #FN
                                np.random.uniform(0.8, 1, size=6)])  #TP

t['y_score3'] = np.concatenate([np.random.uniform(0, 0.2, size=850), #TN
                                np.random.uniform(0.8, 1, size=140), #FP
                                np.random.uniform(0, 0.2, size=2),   #FN
                                np.random.uniform(0.8, 1, size=8)])  #TP



# pdf.metrics.precision_recall(t['y_true'], score=t['y_score1'])

# pdf.metrics.precision_recall(t['y_true'], score=[t['y_score1'], t['y_score2']])

pdf.metrics.precision_recall(t['y_true'], score=[t['y_score1'], t['y_score2'], t['y_score3']], plot=True)


import pandasflow as pdf
import pandas as pd
from sklearn.metrics import roc_curve, roc_auc_score
from matplotlib import pyplot


def roc(target, score, plot=False, marker='', round_=2):
	if type(score) == pd.Series or type(score) == list and len(score) == 1:
		score_ = score if type(score) == pd.Series else score[0]
		score_name = score_.name
		
		fpr, tpr, _ = roc_curve(target, score_)
		pyplot.plot(fpr, tpr, marker=marker, label=score_name)
		
		auc = roc_auc_score(target, score_)
		auc = round(auc, round_)
		print(f"{score_name} auc: ", auc)
	
	if type(score) == list and len(score) > 1:
		for sc in score:
			fpr, tpr, _ = roc_curve(target, sc)
			pyplot.plot(fpr, tpr, marker=marker, label=sc.name)
			
			auc = roc_auc_score(target, sc)
			auc = round(auc, round_)
			print(f"{sc.name} auc: ", auc)
	
	pyplot.xlabel('False Positive Rate')
	pyplot.ylabel('True Positive Rate')
	pyplot.legend(loc = 'lower right')
	
	if plot:
		pyplot.show()


import pandasflow as pdf
from test import df
import pandas as pd



train, test = pdf.split.train_test(df)
print('\n')


train, test = pdf.split.train_test(df, target='Exited')
print('\n')



train, test = pdf.split.train_test(df, target='Exited', stratify='Exited')
print('\n')





import pandas as pd
from sklearn.metrics import f1_score
from tqdm.notebook import tqdm


def best_f1(y_true:pd.Series, score:pd.Series, out:int=1, tqdm_:bool=True):
	thrs = list(score.unique())
	thrs = [max(min(thrs) - 0.000001, 0.000001)] + thrs
	
	df = pd.concat([y_true, score], axis=1)
	
	result = []
	
	if tqdm_: thrs = tqdm(thrs)
	
	for thr in thrs:
		df['y_pred_thr'] = (df[score.name] > thr) * 1
		result.append(
			(
				thr,
				f1_score(df[y_true.name], df['y_pred_thr'])
			)
		)
	
	result = pd.DataFrame(result, columns=['thr', 'f1'])
	result = result.sort_values(by='f1', ascending=False)
	
	print(result.head(min(out, len(result))))












__version__ = '0.4.7'

requirements = ['pandas', 'scikit-learn', 'catboost', 'phik', 'shap', 'tqdm']




from pandasflow.get_import import get_import

from pandasflow.reset_mi import reset_mi



# packages
from pandasflow import split

from pandasflow import metrics







import pandasflow
from setuptools import setup, find_packages


with open("README.md", "r") as readme_file:
    readme = readme_file.read()



setup(
    name="pandasflow",
    version=pandasflow.__version__,
    author="Priboy313",
    author_email="Priboy313@yandex.ru",
    description="A set of custom python modules for friendly workflow on pandas",
    long_description=readme,
    long_description_content_type="text/markdown",
    url="https://github.com/Priboy313/pandasflow",
    packages=find_packages(),
    install_requires=pandasflow.requirements,
    classifiers=[
        "Programming Language :: Python :: 3.10",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
    ],
)


import pandasflow as pdf
import pandas as pd
import numpy as np


df = pd.read_csv('test_data.csv')

train, valid, test = pdf.split.train_valid_test(df)

train_full = pd.concat([train, valid])

model = train_full.groupby('NumOfProducts')['Exited'].mean().reset_index()
model = model.rename({'Exited':'score_prod'}, axis=1)

np.random.seed(42)
noise = np.random.uniform(0, 0.001, size=len(train_full))

train_full = train_full.merge(model, how='left', on='NumOfProducts')
train_full['score_prod'] = train_full['score_prod'] - noise

pdf.metrics.lloss_up(train_full, 'Exited', 'score_prod')


import pandasflow as pdf

import pandas as pd
from sklearn.model_selection import train_test_split

def train_test(
	*arrays,
	test_size=None,
	train_size=0.8,
	random_state=42,
	shuffle=True,
	stratify=None,
	round_=None,
	target=None
):
	"""
	Split arrays or matrices into random train and test subsets.

	Parameters
	----------
	*arrays : sequence of indexables with same length / shape[0]
		Allowed inputs are lists, numpy arrays, scipy-sparse
		matrices or pandas dataframes.

	test_size : float or int, default=None
		If float, should be between 0.0 and 1.0 and represent the proportion
		of the dataset to include in the test split. If int, represents the
		absolute number of test samples. If None, the value is set to the
		complement of the train size. If ``train_size`` is also None, it will
		be set to 0.25.

	train_size : float or int, default=None
		If float, should be between 0.0 and 1.0 and represent the
		proportion of the dataset to include in the train split. If
		int, represents the absolute number of train samples. If None,
		the value is automatically set to the complement of the test size.

	random_state : int, RandomState instance or None, default=None
		Controls the shuffling applied to the data before applying the split.
		Pass an int for reproducible output across multiple function calls.
		See :term:`Glossary <random_state>`.

	round_ : int, RandomState more 0 or None, default=None
		Print the proportion of each array with rounding.
		If None or 0 print all numbers after the decimal point
	"""
	
	df = arrays[0]
	
	_stratify = stratify
	if stratify != None:
		_stratify = df[stratify]
	
	train, test = train_test_split(*arrays, train_size=train_size, random_state=random_state, shuffle=shuffle, stratify=_stratify)
	
	train_pie = len(train) / len(arrays[0])
	test_pie = len(test) / len(arrays[0])
	
	amount_len = len(train) + len(test)
	amount_prop = amount_len / len(arrays[0])
	
	if round_ not in [None, 'n', 'N'] and round_ > 0:
		train_pie = round(train_pie, round_)
		test_pie = round(test_pie, round_)
	
	table = pd.DataFrame({
		'count': [len(train),
			  len(test),
			  '',
			  amount_len,
			  len(arrays[0])],
		
		'pct': [train_pie,
			   test_pie,
			   '',
			   amount_prop,
			   '']
	})
	
	table.index = ['train',
				   'test',
				   '',
				   'Amount',
				   'InitData']
	
	if target != None:
		try:
			if type(target) is str:
				table[target] = [train[target].mean(),
								 test[target].mean(),
								 '',
								 '',
								 df[target].mean()]
			
			
			elif type(target) is list:
				for col in list(target):
					table[col] = [train[col].mean(),
								  test[col].mean(),
								  '',
								  '',
								  df[col].mean()]
		except KeyError:
			raise KeyError(*target)
	
	print(table)
	
	if amount_prop != 1.0 or len(arrays[0]) != amount_len:
		print('---')
		print('Some data is droped')
	
	return train, test








import pandasflow as pdf
from test import df
import pandas as pd




train, valid, test = pdf.split.train_valid_test(df)
print('\n')

train, valid, test = pdf.split.train_valid_test(df, target='Exited')
print('\n')

train, valid, test = pdf.split.train_valid_test(df, target='Exited', stratify='Exited')
print('\n')

train, valid, test = pdf.split.train_valid_test(df, target=['Exited', 'HasCrCard'])
print('\n')

train, valid, test = pdf.split.train_valid_test(df, target=['Exited', 'HasCrCard'], stratify='Exited')
print('\n')

#
# train, valid, test = pdf.split.train_valid_test(df, target=['Surname'])
# print('\n')




import pandas as pd
from sklearn.metrics import log_loss

# Для задач классификации с таргетом 0-1
# Сравнение человеческого baseline с машинным предиктом
# Возвращает две метрики:
# log_loss - чем меньше, тем лучше
# uplift - чем больше, тем лучше


def def_baseline(df, target):
	base = df[target].sum()
	base_pct = df[target].sum() / len(df)
	
	return base, base_pct


def _uplift(df, target, sort_score, pct, baseline=None):
	base, base_pct = def_baseline(df, target)
	if baseline != None:
		print('baseline is not None, but unknown')
	
	_df = df.sort_values(sort_score, ascending=False)
	pct_len = round(len(df) * pct)
	base_found = _df.head(pct_len)[target].sum()

	return base_pct, ((base_found / base) / pct)


def lloss_up(df, target, score, pct=0.2, baseline=None):
	'''Для задач классификации с таргетом 0-1
	Сравнение человеческого baseline с машинным предиктом
	Возвращает две метрики:
	
	log_loss - чем меньше, тем лучше
	uplift - чем больше, тем лучше
	
	Parameters
	----------
	df : pd.DataFrame
		Таблица данных
	
	target : str
		Название колонки с искомым значением
	
	score : str
		Название колонки с предиктом машинного обучение
		Результат работы, который и сравниваем
	
	pct : float, default = 0.2
		Процент колонки предикта, из которого и считаем uplift
	
	baseline : None, def, int, float. default = None
		Худший вариант предсказания, базово - среднееарифметическое
		Лучше не трогать и оно посчитается от target
	'''

	lloss = log_loss(df[target], df[score])
	base, up = _uplift(df, target, score, pct, baseline=baseline)
	
	table = pd.DataFrame()
	table.index = ['baseline', 'log_loss', 'uplift']
	table[' '] = [base, lloss, up]
	
	print(table)
	print()
	return base, lloss, up